package data;

import user.User;

import java.io.*;
import java.util.LinkedHashMap;
import java.util.Properties;

public class LocalDataBase {
    private LinkedHashMap<String, User> userMap;//存储用户数据的表
    private LinkedHashMap<String, String[]> songMap;// 歌曲单 键值为歌手name
    private String userMapUrl;
    private String songMapUrl;
    private static LocalDataBase localDataBase;// 单例数据库对象

    /**
     * 无参构造函数
     */
    private LocalDataBase() {
        userMap = new LinkedHashMap<>();
        songMap = new LinkedHashMap<>();
    }


    /**
     * @return 外部访问单例对象
     */
    public static LocalDataBase getLocalDataBase() {
        if (localDataBase == null) {
            localDataBase = new LocalDataBase();
        }
        return localDataBase;
    }


    /**
     * 用父类参数定义的多态元素添加方式
     * @param user 使用者对象
     * @return 返回是否添加成功
     */
    public boolean putUser(User user) {
        try {
            userMap.put(user.getId(), user);
            outputDataBase();
        }catch (NullPointerException e){
            e.printStackTrace();
            return false;
        }
        return true;
    }
    public void loadDataBase() {
        loadConfig();// 加载配置属性
        userMap.clear();// 清空
        Object o = deserializeObject(userMapUrl);
        if(o instanceof LinkedHashMap) {
            userMap = (LinkedHashMap<String, User>) o;
        }
    }
    public void outputDataBase() {
        serializeObject(userMap,userMapUrl);
    }
    public void refreshDataBase() {
        //刷新即写出去再读入，如果是数据库就不用写出去，因为是一直同步的，现在相当于在内存里操作，没有写出同步。
        outputDataBase();
        loadDataBase();
    }

    /**
     * 加载数据库路径配置
     */
    private void loadConfig() {
        Properties properties = new Properties();
        try {
            FileInputStream fileInputStream = new FileInputStream("./src/config.properties");
            properties.load(fileInputStream);
            fileInputStream.close();

            userMapUrl = properties.getProperty("database.userMapUrl");
//            songMapUrl = properties.getProperty("database.songMapUrl");
            System.out.println("> 数据库参数配置成功，正在导入...");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * @param object 集合对象
     * @param filename 数据库路径
     */
    private void serializeObject(Object object, String filename) {
        try {
            //定义文件流
            FileOutputStream fileOut = new FileOutputStream(filename);
            ObjectOutputStream objOut = new ObjectOutputStream(fileOut);

//            LinkedHashMap<Integer, User> map = (LinkedHashMap<Integer, User>) object;//不用检查，一定是这个类型
            objOut.writeObject(object);
            objOut.close();
            fileOut.close();
            System.out.println("> 对象已序列化并保存到文件数据库");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private Object deserializeObject(String filename) {
        Object object = null;
        try {
            FileInputStream fileIn = new FileInputStream(filename);
            ObjectInputStream objIn = new ObjectInputStream(fileIn);
            object = objIn.readObject();
            objIn.close();
            fileIn.close();
            System.out.println("> 从文件数据库反序列化对象完成！");
        } catch (IOException | ClassNotFoundException e) {
            e.printStackTrace();
        }
        return object;
    }

    public <T extends Iterable<?>> void printMap(T o){
//        for(Map.Entry<?, ?> entry: map.entrySet()){
//             System.out.println(entry.getValue().toString());
//        }
        for(Object element: o){
            System.out.println(element);
        }
    }
    public String getUserMapUrl() {
        return userMapUrl;
    }
    public LinkedHashMap<String, User> getUserMap() {
        return userMap;
    }

    public void setUserMapUrl(String userMapUrl) {
        this.userMapUrl = userMapUrl;
    }

    public LinkedHashMap<String, String[]> getSongMap() {
        return songMap;
    }

    public String getSongMapUrl() {
        return songMapUrl;
    }

    public void setSongMapUrl(String songMapUrl) {
        this.songMapUrl = songMapUrl;
    }
}
